"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const RESTClient_1 = require("./RESTClient");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const crypto_1 = __importDefault(require("crypto"));
const ENV_1 = require("./ENV");
const config_1 = require("./config");
const FileTools_1 = require("./FileTools");
exports.attemptToUpdateSticker = () => __awaiter(void 0, void 0, void 0, function* () {
    const { sticker: { sticker: currentSticker }, } = config_1.getTheme();
    const remoteStickerUrl = `${ENV_1.VSCODE_ASSETS_URL}${stickerPathToUrl(currentSticker)}`;
    const remoteWallpaperUrl = `${ENV_1.BACKGROUND_ASSETS_URL}${wallpaperPathToUrl(currentSticker)}`;
    const localStickerPath = exports.resolveLocalStickerPath(currentSticker);
    const localWallpaperPath = exports.resolveLocalWallpaperPath(currentSticker);
    yield Promise.all([
        attemptToUpdateAsset(remoteStickerUrl, localStickerPath),
        attemptToUpdateAsset(remoteWallpaperUrl, localWallpaperPath),
    ]);
    return {
        stickerDataURL: createCssDokiAssetUrl(localStickerPath),
        wallpaperURL: createCssDokiAssetUrl(localWallpaperPath),
    };
});
function attemptToUpdateAsset(remoteStickerUrl, localStickerPath) {
    return __awaiter(this, void 0, void 0, function* () {
        if (yield shouldDownloadNewAsset(remoteStickerUrl, localStickerPath)) {
            yield installAsset(remoteStickerUrl, localStickerPath);
        }
    });
}
const fetchRemoteChecksum = (remoteAssetUrl) => __awaiter(void 0, void 0, void 0, function* () {
    const checksumUrl = `${remoteAssetUrl}.checksum.txt`;
    console.log(`Fetching resource checksum: ${checksumUrl}`);
    const checkSumInputStream = yield RESTClient_1.performGet(checksumUrl);
    return checkSumInputStream.setEncoding("utf8").read();
});
exports.resolveLocalStickerPath = (currentSticker) => {
    const safeStickerPath = stickerPathToUrl(currentSticker);
    return path_1.default.join(config_1.configDirectory, "stickers", safeStickerPath);
};
exports.resolveLocalWallpaperPath = (currentSticker) => {
    const safeStickerPath = wallpaperPathToUrl(currentSticker);
    return path_1.default.join(config_1.configDirectory, "wallpapers", safeStickerPath);
};
const createCssDokiAssetUrl = (localAssetPath) => {
    return `file://${cleanPathToUrl(localAssetPath)}`;
};
function cleanPathToUrl(stickerPath) {
    const unEncodedUrl = stickerPath.replace(/\\/g, "/");
    const encodedUrl = encodeURI(unEncodedUrl).replace(/[!'()*]/g, escape);
    return encodedUrl;
}
function stickerPathToUrl(currentSticker) {
    const stickerPath = currentSticker.path;
    return cleanPathToUrl(stickerPath);
}
function wallpaperPathToUrl(currentSticker) {
    const stickerPath = `/` + currentSticker.name;
    return cleanPathToUrl(stickerPath);
}
function createChecksum(data) {
    return crypto_1.default.createHash("md5").update(data).digest("hex");
}
const calculateFileChecksum = (filePath) => {
    const fileRead = fs_1.default.readFileSync(filePath);
    return createChecksum(fileRead);
};
const fetchLocalChecksum = (localSticker) => __awaiter(void 0, void 0, void 0, function* () {
    return fs_1.default.existsSync(localSticker)
        ? calculateFileChecksum(localSticker)
        : "File not downloaded, bruv.";
});
const shouldDownloadNewAsset = (remoteAssetUrl, localStickerPath) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const remoteChecksum = yield fetchRemoteChecksum(remoteAssetUrl);
        const localChecksum = yield fetchLocalChecksum(localStickerPath);
        return remoteChecksum !== localChecksum;
    }
    catch (e) {
        console.error("Unable to check for updates", e);
        return false;
    }
});
const downloadRemoteAsset = (remoteAssetUrl, localDestination) => __awaiter(void 0, void 0, void 0, function* () {
    FileTools_1.createParentDirectories(localDestination);
    console.log(`Downloading remote asset: ${remoteAssetUrl}`);
    const stickerInputStream = yield RESTClient_1.performGet(remoteAssetUrl);
    console.log("Remote asset Downloaded!");
    fs_1.default.writeFileSync(localDestination, stickerInputStream.read());
});
function installAsset(remoteAssetUrl, localAssetPath) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield downloadRemoteAsset(remoteAssetUrl, localAssetPath);
            return true;
        }
        catch (e) {
            console.error(`Unable to install asset ${remoteAssetUrl}!`, e);
        }
        return false;
    });
}
//# sourceMappingURL=StickerUpdateService.js.map