"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const themeTools_1 = require("./themeTools");
const syntax_1 = require("./syntax");
const css_1 = require("./css");
const path_1 = __importDefault(require("path"));
const fs_1 = __importStar(require("fs"));
const os_1 = __importDefault(require("os"));
const FileTools_1 = require("./FileTools");
const vm_1 = __importDefault(require("vm"));
const applicationDirectory = process.env.XDG_CONFIG_HOME !== undefined
    ? path_1.default.join(process.env.XDG_CONFIG_HOME, "hyper")
    : process.platform == "win32"
        ? path_1.default.join(process.env.APPDATA || "", "Hyper")
        : os_1.default.homedir();
exports.configDirectory = path_1.default.resolve(applicationDirectory, ".doki-theme-hyper-config");
const configFile = path_1.default.resolve(exports.configDirectory, ".hyper.doki.config.json");
const hyperConfigFile = path_1.default.resolve(applicationDirectory, ".hyper.js");
exports.DEFAULT_THEME_ID = "5ca2846d-31a9-40b3-8908-965dad3c127d"; // Rimiru
exports.DEFAULT_CONFIGURATION = {
    themeId: exports.DEFAULT_THEME_ID,
    showSticker: true,
    showWallpaper: true,
    stickerType: themeTools_1.StickerType.DEFAULT,
    useFonts: false,
};
exports.extractConfig = () => {
    if (!fs_1.default.existsSync(configFile)) {
        FileTools_1.createParentDirectories(configFile);
        fs_1.default.writeFileSync(configFile, JSON.stringify(exports.DEFAULT_CONFIGURATION), "utf8");
        return exports.DEFAULT_CONFIGURATION;
    }
    return JSON.parse(fs_1.default.readFileSync(configFile, "utf8"));
};
const defaultConfig = {
    dokiSettings: {
        backgrounds: {}
    }
};
const _extract = (script) => {
    const module = {};
    script === null || script === void 0 ? void 0 : script.runInNewContext({ module });
    if (!module.exports) {
        throw new Error('Error reading configuration: `module.exports` not set');
    }
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return module.exports;
};
const _syntaxValidation = (cfg) => {
    try {
        return new vm_1.default.Script(cfg, { filename: '.hyper.js', displayErrors: true });
    }
    catch (_err) {
        const err = _err;
        console.error(`Error loading config: ${err.name}`, `${err}`, { error: err });
    }
};
const _extractDefault = (cfg) => {
    return _extract(_syntaxValidation(cfg));
};
// there isn't a good way to access the application
// configuration (.hyper.js), so will just be
// reading the contents of the file and loading the
// js, and passing the doki confi
exports.extractHyperConfig = () => {
    if (!fs_1.default.existsSync(hyperConfigFile)) {
        return defaultConfig;
    }
    const extractDefault = _extractDefault(fs_1.readFileSync(hyperConfigFile, 'utf8'));
    return (extractDefault === null || extractDefault === void 0 ? void 0 : extractDefault.config) || defaultConfig;
};
exports.saveConfig = (dokiConfig) => {
    fs_1.default.writeFileSync(configFile, JSON.stringify(dokiConfig), "utf8");
};
function getCorrectSticker(theme, stickerType) {
    const defaultSticker = theme.stickers.default;
    return stickerType === themeTools_1.StickerType.SECONDARY
        ? theme.stickers.secondary || defaultSticker
        : defaultSticker;
}
exports.getCorrectSticker = getCorrectSticker;
exports.getTheme = () => {
    const hyperDokiConfig = exports.extractConfig();
    const theme = themeTools_1.getThemeByName(hyperDokiConfig.themeId);
    const stickerType = hyperDokiConfig.stickerType;
    const sticker = getCorrectSticker(theme, stickerType);
    return {
        theme,
        sticker: {
            sticker,
            type: stickerType,
        },
    };
};
const getExtraSettings = () => {
    return exports.extractConfig().useFonts
        ? {
            fontFamily: '"Victor Mono", Menlo, "DejaVu Sans Mono", Consolas, "Lucida Console", monospace',
            fontWeight: "italic",
        }
        : {};
};
exports.decorateConfig = (config) => {
    const { theme: dokiTheme, sticker } = exports.getTheme();
    const syntax = syntax_1.constructSyntax(dokiTheme);
    const css = css_1.constructCSS(dokiTheme, sticker, config);
    return Object.assign({}, config, syntax, Object.assign(Object.assign({}, getExtraSettings()), { termCSS: config.termCSS || "", css: `${config.css || ""}
    ${css}
    ` }));
};
//# sourceMappingURL=config.js.map