"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const settings_1 = require("./settings");
const config_1 = require("./config");
const DokiThemeDefinitions_1 = __importDefault(require("./DokiThemeDefinitions"));
const decorator_1 = require("./decorator");
var SystemState;
(function (SystemState) {
    SystemState[SystemState["LIGHT"] = 0] = "LIGHT";
    SystemState[SystemState["DARK"] = 1] = "DARK";
})(SystemState || (SystemState = {}));
exports.THEME_STATE = "dokiThemeState";
const themeNameToDefinition = Object.values(DokiThemeDefinitions_1.default).reduce((accum, def) => {
    accum[def.information.name] = def;
    return accum;
}, {});
function applySystemState(state) {
    var _a, _b;
    const hyperConfig = config_1.extractHyperConfig();
    if ((_b = (_a = hyperConfig.dokiSettings) === null || _a === void 0 ? void 0 : _a.systemMatch) === null || _b === void 0 ? void 0 : _b.enabled) {
        const previousState = state[exports.THEME_STATE] || {};
        const themeKey = previousState.systemState === SystemState.DARK ?
            hyperConfig.dokiSettings.systemMatch.darkTheme || 'Zero Two Dark' :
            hyperConfig.dokiSettings.systemMatch.lightTheme || 'Zero Two Light';
        const activeThemeDef = themeNameToDefinition[themeKey] || themeNameToDefinition[DokiThemeDefinitions_1.default[config_1.DEFAULT_THEME_ID].information.name];
        const themeState = Object.assign(Object.assign({}, previousState), { activeTheme: activeThemeDef });
        config_1.saveConfig(Object.assign(Object.assign({}, config_1.extractConfig()), { themeId: activeThemeDef.information.id }));
        return state.set(exports.THEME_STATE, themeState);
    }
    else {
        return state;
    }
}
const reducer = (state, action) => {
    switch (action.type) {
        case settings_1.SET_THEME: {
            const previousState = state[exports.THEME_STATE] || {};
            const themeState = Object.assign(Object.assign({}, previousState), { activeTheme: action.payload, currentSticker: {
                    type: previousState.currentSticker.type,
                    sticker: config_1.getCorrectSticker(action.payload, previousState.currentSticker.type),
                } });
            config_1.saveConfig(Object.assign(Object.assign({}, config_1.extractConfig()), { themeId: action.payload.information.id }));
            return state.set(exports.THEME_STATE, themeState);
        }
        case settings_1.SET_STICKER_TYPE: {
            const previousState = state[exports.THEME_STATE] || {};
            const themeState = Object.assign(Object.assign({}, previousState), { currentSticker: {
                    type: action.payload,
                    sticker: config_1.getCorrectSticker(previousState.activeTheme, action.payload),
                } });
            return state.set(exports.THEME_STATE, themeState);
        }
        case settings_1.SYSTEM_THEME_CHANGED: {
            const isWindowDark = action.payload.isDark;
            const previousThemeState = state[exports.THEME_STATE] || {};
            const newThemeState = Object.assign(Object.assign({}, previousThemeState), { systemState: isWindowDark ? SystemState.DARK : SystemState.LIGHT });
            return applySystemState(state.set(exports.THEME_STATE, newThemeState));
        }
        // this forces theme updates on config change.
        case decorator_1.CONFIG_RELOAD: {
            return applySystemState(state);
        }
        case settings_1.TOGGLE_STICKER: {
            const previousState2 = state[exports.THEME_STATE] || {};
            const themeState = Object.assign(Object.assign({}, previousState2), { showSticker: !state[exports.THEME_STATE].showSticker });
            return state.set(exports.THEME_STATE, themeState);
        }
        case settings_1.TOGGLE_WALLPAPER: {
            const previousState2 = state[exports.THEME_STATE] || {};
            const themeState = Object.assign(Object.assign({}, previousState2), { showWallpaper: !state[exports.THEME_STATE].showWallpaper });
            return state.set(exports.THEME_STATE, themeState);
        }
        case "INIT": {
            const { theme, sticker } = config_1.getTheme();
            const dokiThemeConfig = config_1.extractConfig();
            const themeState = {
                activeTheme: theme,
                currentSticker: sticker,
                showSticker: dokiThemeConfig.showSticker,
                showWallpaper: dokiThemeConfig.showWallpaper,
                systemState: SystemState.DARK,
            };
            return state.set(exports.THEME_STATE, themeState);
        }
        default: {
            return state;
        }
    }
};
exports.default = reducer;
//# sourceMappingURL=reducer.js.map