"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const DokiThemeDefinitions_1 = __importDefault(require("./DokiThemeDefinitions"));
const config_1 = require("./config");
const electron_1 = require("electron");
const path_1 = __importDefault(require("path"));
const StickerUpdateService_1 = require("./StickerUpdateService");
const themeTools_1 = require("./themeTools");
exports.SET_THEME = "SET_THEME";
exports.SET_STICKER_TYPE = "SET_STICKER_TYPE";
exports.SYSTEM_THEME_CHANGED = "SYSTEM_THEME_CHANGED";
exports.TOGGLE_STICKER = "TOGGLE_STICKER";
exports.TOGGLE_WALLPAPER = "TOGGLE_WALLPAPER";
exports.TOGGLE_FONT = "TOGGLE_FONT";
exports.STICKER_UPDATED = "STICKER_UPDATED";
function attemptToDoStickerStuff(focusedWindow) {
    StickerUpdateService_1.attemptToUpdateSticker().then(() => {
        focusedWindow.rpc.emit(exports.STICKER_UPDATED);
    });
    setTimeout(() => {
        // triggers event loop to continue download?
        focusedWindow.rpc.emit("refresh");
    }, 500);
}
const stickerTypeMenus = [
    {
        name: "Primary",
        type: themeTools_1.StickerType.DEFAULT,
    },
    {
        name: "Secondary",
        type: themeTools_1.StickerType.SECONDARY,
    },
].map(({ name, type }) => {
    return {
        label: name,
        click: (_, focusedWindow) => __awaiter(void 0, void 0, void 0, function* () {
            config_1.saveConfig(Object.assign(Object.assign({}, config_1.extractConfig()), { stickerType: type }));
            focusedWindow.rpc.emit(exports.SET_STICKER_TYPE, type);
            attemptToDoStickerStuff(focusedWindow);
        }),
    };
});
const themes = Object.values(DokiThemeDefinitions_1.default)
    .sort((def1, def2) => def1.information.name.localeCompare(def2.information.name))
    .map((dokiDefinition) => {
    return {
        label: dokiDefinition.information.name,
        click: (_, focusedWindow) => __awaiter(void 0, void 0, void 0, function* () {
            focusedWindow.rpc.emit(exports.SET_THEME, dokiDefinition);
            attemptToDoStickerStuff(focusedWindow);
        }),
    };
});
exports.VERSION = "v78.0-1.1.0";
const icon = path_1.default.resolve(__dirname, "..", "assets", "Doki-Theme.png");
const showAbout = () => {
    const appName = "Doki Theme";
    electron_1.dialog.showMessageBox({
        title: `About ${appName}`,
        message: `${appName} ${exports.VERSION}`,
        detail: `Thanks for downloading The Doki Theme for Hyper!`,
        buttons: ["K Thx"],
        // @ts-ignore
        icon,
    });
};
const getAboutMenu = () => {
    if (process.platform !== "darwin") {
        return {
            label: "About Plugin",
            click() {
                showAbout();
            },
        };
    }
    return {
        label: "About Plugin",
        submenu: [
            {
                label: `Version ${exports.VERSION}`,
            },
        ],
    };
};
exports.default = (providedMenu) => {
    const menuItem = {
        id: "Doki-Theme",
        label: "Doki-Theme Settings",
        commandId: 1969,
        checked: false,
        enabled: true,
        role: "help",
        submenu: [
            {
                id: "Themes",
                label: "Themes",
                commandId: 1969,
                checked: false,
                enabled: true,
                role: "help",
                submenu: themes,
            },
            {
                id: "StickerType",
                label: "Sticker Type",
                commandId: 1985,
                checked: false,
                enabled: true,
                submenu: stickerTypeMenus,
            },
            {
                label: "Toggle Sticker",
                click: (_, focusedWindow) => __awaiter(void 0, void 0, void 0, function* () {
                    const savedConfig = config_1.extractConfig();
                    const showSticker = !savedConfig.showSticker;
                    focusedWindow.rpc.emit(exports.TOGGLE_STICKER);
                    config_1.saveConfig(Object.assign(Object.assign({}, savedConfig), { showSticker }));
                }),
            },
            {
                label: "Toggle Wallpaper",
                click: (_, focusedWindow) => __awaiter(void 0, void 0, void 0, function* () {
                    const savedConfig = config_1.extractConfig();
                    const showWallpaper = !(savedConfig.showWallpaper || savedConfig.showWallpaper == undefined);
                    focusedWindow.rpc.emit(exports.TOGGLE_WALLPAPER);
                    config_1.saveConfig(Object.assign(Object.assign({}, savedConfig), { showWallpaper }));
                }),
            },
            {
                label: "Toggle Fonts",
                click: (_, focusedWindow) => __awaiter(void 0, void 0, void 0, function* () {
                    const savedConfig = config_1.extractConfig();
                    const useFonts = !savedConfig.useFonts;
                    config_1.saveConfig(Object.assign(Object.assign({}, savedConfig), { useFonts }));
                    focusedWindow.rpc.emit(exports.TOGGLE_FONT);
                }),
            },
            getAboutMenu(),
            {
                label: "View ChangeLog",
                click: () => __awaiter(void 0, void 0, void 0, function* () {
                    // eslint-disable-next-line @typescript-eslint/no-var-requires
                    const { shell } = require("electron");
                    yield shell.openExternal("https://github.com/doki-theme/doki-theme-hyper/blob/master/CHANGELOG.md");
                }),
            },
        ],
    };
    return [...providedMenu, menuItem];
};
//# sourceMappingURL=settings.js.map